%
% ################          cEKF Correction        ####################
%
% ##################      4 CYLINDRICAL TANKS      #################### 
%
% By Giovani Tonel (giotonel@enq.ufrgs.br) - May 2007



% *********************************************************************************
%
%                          cEKF OPTIMIZATION
%
% w_k|k	=	x_k|k 		-	x_k|k-1		
% v_k|k	=	g(x_k|k) 	- 	y_k			

Tzero	= 	zeros(ns,nms);
T 			= 	inv([(Pp) Tzero; Tzero' (R)]);

% --------------- To eliminate the problems from the inversion matrix -------------
T 			= ( T + T' )/2;   			  
% ---------------------------------------------------------------------------------
     
l 			= 	zeros(ns+nms,1);

     
% ------------------------- Restrition of Equality  -------------------------------
%
% The optimization problem is subject to the following equality equations:
% x_k|k		=	x_k|k-1 	+	w_k|k		-->
% y_k		=	g(x_k|k) 	+ 	v_k|k		-->
% 
% so we can get a unique equality equation, by replacing x_k|k in the second equation:
% 
% y_k		=	g(x_k|k-1 	+	w_k-1|k) 	+ 	v_k|k 
% If g is linear:
% y_k		= g(x_k|k-1) 	+	ewv_k|k --> ewv_k|k	=	y_k	- g(x_k|k-1)
% by the solution of the optimization problem we'll find the minimum value of
% ewv_k|k = sum{w_k-1|k, 	v_k|k }

yc			=	smfun(xp, parpro);          
G			= 	Gmatrix(xp,parpro);

if flag1~=3,
   beq 	= 	ymea-yc ;						% beq in relation to states measured
else
   beq 	= 	G*ymea-xp;					% beq in relation to Fout measured
end


Aeq 		= 	[G G(1:nms,1:nms)]; 	% Aeq is a identity matrix if G is linear

% ---------------------------------------------------------------------------------

% ---------------------------------- Bounds ---------------------------------------

% "bounds.m" contains the boundaries for the states variables , and not the boundaries
% to the measured states disturbance variables, that are used to solve the MHE problem.

% bounds;
% Lb		=	[];
% Ub		=	[];
% --------------------------------------------------------------------------------
     
% ------------------- Initial guess for the solver optimization -------------------
wv0			=	zeros(ns+nms,1); 
% ---------------------------------------------------------------------------------

% ------------------- Set/run the optimization problem  ---------------------------
op			=	optimset('LargeScale','off');
     
% x		= 	quadprog(T,l,A,b,Aeq,beq,LB,UB,X0,OPTIONS)
wvopt 	= 	quadprog(T,l,[],[],Aeq,beq,[],[],wv0,op); 
% ---------------------------------------------------------------------------------
% *                                                                               *
% *                                                                               *
% *********************************************************************************
     
     
%__________________________________________________________________________________
%
%                         EKF Comparison 
%     
% K				=	(Pest*G')/(G*Pest*G'+R);
% xcor			= 	xest+K*(ymea-G*xest); 
% id				=	find(xcor<0);	% remove the negative values of x correction matrix, 
%            						 	% in order to avoid that the filter lose in the initialization
% id				=	[];
% xcor(id)	=	zeros(size(id));
% I				=	eye(ns,ns);
% Pcor			=	(I-K*G)*Pest*(I-K*G)'+K*R*K';
% Pcor			=	(I-K*G)*Pest;
%__________________________________________________________________________________

     
     
% -------------------------   Optimization results --------------------------------
wopt		=	wvopt(1:ns);				
vopt		=	wvopt(ns+1:ns+nms);

% ---------------------------------------------------------------------------------
         
     
% -------------------------  Correcting States ------------------------------------
% x_k|k 	= x_k|k-1 	+ 	w_k|k
xopt			=	xp 			+ 	wopt;
x_kk			=	xopt;
% ---------------------------------------------------------------------------------

     
% -----------------------    Correcting F Jacobian Matrix -------------------------
F 			= Fmatrix(x_kk,parpro);
% ---------------------------------------------------------------------------------

% ------------------------  Correcting G Jacobian Matrix --------------------------
G			= Gmatrix(x_kk,parpro);
% ---------------------------------------------------------------------------------
     
% ------------------------- Up-dating P gain matrix  ------------------------------
% There are some forms of the update the P matrix, for this case we decided use this:
% Pcor 		= Q + F*Ppred*F'-F*Ppred*G'*inv(G*Ppredt*G'+R)*G*Ppred*F';

Pcor 		= Q + F*Pp*F' - F*Pp*G'*inv(G*Pp*G' + R)*G*Pp*F';
% ---------------------------------------------------------------------------------
