%
% ################          cEKF Prediction        ####################
%
% ##################      4 CYLINDRICAL TANKS      #################### 
%
% By Giovani Tonel (giotonel@enq.ufrgs.br) - May 2007


%__________________________________________________________________________________ 

switch namesf
case 1
   %
   % Setting parameters which came from the 'MHE Correction Phase', and they will be
   % used on the 'MHE Prediction Phase' (below lines).
   
   ti 				= 	t-(N)*dt		;						% t(k-N)	
   tf				=	t-(N-1)*dt	;						% t(k-N+1)
   delt				=	[ti , tf]		;						% [initial t, final t]
   Pxcor  		= [xkNcor(:); PkNcor(:)];   	% [x{k-N|k}cor, P{k-N|k}cor]
   
   %__________________________________________________________________________________ 
   %
   % Runs 'Pxpred' function to obtain the P_k-N+1|k value from {x_k-N|k}cor and
   % {P_k-N|k}cor
       
   [T,Y]			=	ode45('Pxpred',delt, Pxcor, [], Q, parpro);
   
   x_kN1			=	Y(end,1:ns)';  												% xest= x estimated = x_k-N+1|k
   P_kN1			=	reshape(Y(end,(ns+1):(ns+ns^2)),ns,ns);		% Pest= P estimated = P_k-N+1|k
   
   % x{ k-N+1|k}cor - x{ k-N+1|k}pred
   % disp('xopt(:,1)- x_kN1 difference: ')
   
   % xopt(:,1)- x_kN1
   
   % Note: this step is the properly MHE prediction phase.
   %__________________________________________________________________________________
   %
   % Runs 'pmfun' function to obtain (estimate) the {x_k+1|k} value from {x_k|k}cor
   
   ti 				= 	t					;					% t(k)	
   tf				=	t + dt			;					% t(k+1)
   delt				=	[ti , tf]		;					% [initial t, final t]
   
   parpro.umv	=	umv 				;					% u{k|k}
   
   Ypred			= 	pmfun(x_kk, delt, parpro);
      
   x_k1				=	Ypred(1:ns)';					% size= (ns, 1)
   %__________________________________________________________________________________
   
   
   
   

   
case 2
   %
   % Setting parameters which came from the 'cEKF Correction Phase', and they will be 
   % used on the 'cEKF Prediction Phase'
   
   ti 				= 	t					;					% t(k)	
   tf				=	t + dt			;					% t(k+1)
   delt				=	[ti , tf]		;					% [initial t, final t]
   x_cor 			= 	[x_kk(:); Pcor(:)];   	% [x{k|k}cor, P{k|k}cor]
   
   
   %__________________________________________________________________________________ 
   
   [T,Y]			=	ode45('Pxpred',delt, x_cor, [], Q, parpro);
   
   x_k1				=	Y(end,1:ns)';  												% xest= x estimated = x_k+1|k
   P_k1				=	reshape(Y(end,(ns+1):(ns+ns^2)),ns,ns);		% Pest= P estimated = P_k+1|k
   
   %__________________________________________________________________________________ 
end
  

% Note: "end" gets the last calculated value(between tk and tk+1) by the ode integrator
