function Gm = Gmatrix(x,parpro) 
%                                           
% ######################      JACOBIAN MATRIX G  ######################################
% ######################                         ######################################       
% ######################     4 CYLINDRICAL TANKS ######################################
%
%
%	By Giovani Tonel (giotonel@enq.ufrgs.br) - April 2007			     
%
%

%_________________________________________________________________________________
%
% Process Parameters


% S-function name
namesf		= 3;

%  Convert field to variable
par_pro;

%_________________________________________________________________________________




% 
% 
% - G MATRIX 
% Ex.: suppose that the measured variables are the output flow Fout1 and Fout2 of the tank 1 and 2 
% respectively, so the relation between measured variables with the states l1 and l2 (l=level 
% of the tank) will be Fout(1,2)= Cv*sqrt(l(1,2)) and the G matrix will be the :
% 
% 				
%
% 
% 				G		=	[	dy1/dx1		dy1/dx2		dy1/dx3		dy1/dx4;
%								dy2/dx1		dy2/dx2		dy2/dx3		dy2/dx4]
%
% 		  						
%
% where 	y		= g(x(t)) + v(t)
%				y1		= 	Fout1	= Cv*sqrt(l1));	 	--> l1= x1
%				y2		= 	Fout2	= Cv*sqrt(l2));	 	--> l2= x2
%				y3		= 	--> not measured
%				y4		=	--> not measured
%				
% 
% The g matrix is a matrix that links the inputs with the outputs, y= g(x(t))+ v(t). 
% 
% Ex.: xest(2,1) <-- C(2,4)*xest(4,1)     
%
% 			C	=	[	1 0 0 0;
%     				0 1 0 0 ];



%_________________________________________________________________________________
%
%                                   Choosing G matrix
%_________________________________________________________________________________ 

% (1) 	--> linear system
% (2) 	--> nonlinear system
% (3) 	--> nonlinear system (modified)

switch flag1
case 1   												% Note: for this case the measured variables of the 
                          					% process are the own states.
   if ns==nms,
      Gm	=	eye(ns);
   else
      Gm	=	eye(nms,ns);
   end
   
case 2
   
   Cv(1)		=	a(1)*sqrt(2*g);      	% Cv= R= a/A= (Fout/sqrt(h))ss --> ss= steady state
   Cv(2)		=	a(2)*sqrt(2*g);			% m^2.5/s 
   
   G11	=	Cv(1)/(sqrt(xest(1)));
   G22	=	Cv(2)/(sqrt(xest(2)));
   
   Gm	=	[	
      G11 	0 		0 		0	;
      0 		G22 	0 		0]	;
   
case 3
   Cv(1)		=	a(1)*sqrt(2*g);      	% Cv= R= a/A= (Fout/sqrt(h))ss --> ss= steady state
   Cv(2)		=	a(2)*sqrt(2*g);			% m^2.5/s 
   
   G11	=	2*(sqrt(xest(1))/Cv(1));
   G22	=	2*(sqrt(xest(2))/Cv(2));
   
   Gm	=	[	
      G11 	0		;
      0 		G22	;
      0			0		;
      0			0	]	;
end


return

%_________________________________________________________________________________ 


   

