function [sys,x0,str,ts] = MHE(t,x,u,flag, parmhe, parpro)
%
% ##################         MHE S-FUNCTION               ##################
%
% ##################   MOVING HORIZON ESTIMATOR (MHE)     ##################
%
% ##################        4 CYLINDRICAL TANKS           ################## 
%___________________________________________________________________________
%___________________________________________________________________________
%
%             Giovani Tonel (giotonel@enq.ufrgs.br) - April 2007
%___________________________________________________________________________
%___________________________________________________________________________

%__________________________________
%

% S-function name
namesf		= 1;

%  Convert field to variable
par_pro;
%__________________________________


%___________________________________________________________________________________
%
%                                Beginning th S-function
%___________________________________________________________________________________

switch flag,
   

  %%%%%%%%%%%%%%%%%%
  % Initialization %
  %%%%%%%%%%%%%%%%%%
case 0,
   
    sys = [0,      					% number of continuous states
           ns+ns^2,      		% number of discrete states
           ns,      				% number of outputs
           nms+num,      		% number of inputs
           0,      					% reserved must be zero
           1,      					% direct feedthrough flag
           1];     					% number of sample times
        
    x0  = [x0(:); P0(:)];  	% initialize the initial conditions
    str = [];      					% str is always an empty matrix
    ts  = [dt 0];   				% sample time: [period, offset]
    
    
    
  %%%%%%%%%%%%%%%
  % Derivatives %
  %%%%%%%%%%%%%%%
case 1,
   
   sys=[];
   % If you put the diferential equations here, the matlab/simulink solve them,
   % using the default solver (ode45).    
   
   
  %%%%%%%%%%
  % Update %
  %%%%%%%%%%
case 2,
   

     % If the model has continuous states, ode45 is used. ode45 is an excellent general 
     % purpose solver. However, if you know that your system is stiff and if ode45 is not 
     % providing acceptable results, try ode15s.
     
     % 
     xp			= 	x(1:ns,1);												% xp = Predicte x{k+1|k-1} on tk-1
     Pp			=	reshape(x((ns+1):(ns+ns^2)),ns,ns);		% Pp = Predicte P{k-N|k-1} on t_k-1
     
          
     %__________________________________________________________________________________
     %
     %                             INPUT PROCESS MEASUREMENT (IPM)
     %
     %
     % 			Filter inputs:									Filter outputs:
     %
     % 		u(1) <-- F1	(feed flow)	;				% y(1) <-- h1 (filtered)	;
     % 		u(2) <-- F2	(feed flow)	;				% y(2) <-- h2 (filtered)	;
     % 		u(3) <-- X1	(fr. flow)	;				% y(3) <-- h3 (estimated)	;
     % 		u(4) <-- X2	(fr. flow)	;				% y(4) <-- h4 (estimated)	;
     % 		u(5) <-- h1 (measured)	;					
     % 		u(6) <-- h2 (measured)	;
     
     ipm;   
         
     %__________________________________________________________________________________
  
            
     
     %__________________________________________________________________________________
     %
     %                         Get parameters/values from the buffer
     % We are using the simulinks blocks to record or to buffer the data obtained from the
     %	MHE algorithm, but we have concience that it is not right way to do it. If you have
     % a better source, like store the data in a file, you can put it here. 
     
     xfil 		= 	get_param('MHE_CEKF/x_fil', 'Value');
     xbuf 		= 	get_param('MHE_CEKF/x_smt', 'Value');
     umvb			= 	get_param('MHE_CEKF/u_mvb', 'Value');
     ybuf 		= 	get_param('MHE_CEKF/y_mea', 'Value');
     
     xfil 		=	str2num(xfil);
     xbuf 		=	str2num(xbuf);
     umvb 		=	str2num(umvb);
     ybuf 		=	str2num(ybuf);
     
     xfil			=	reshape(xfil, ns , 1  );		% size= (ns  , 1   )
     xbuf			=	reshape(xbuf, ns , N+1);		% size= (ns  , N+1 )
     umvb			=	reshape(umvb, num, N+2);		% size= (num , N+2 )
     ybuf			=	reshape(ybuf, nms, N+1);		% size= (nms , N+1 )
     
         
     %__________________________________________________________________________________
     %
     % Replacing the y buffer with the new measurements
     % Replacing the manipulated variables (F1, F2, X1, X2) with the new values
     
     for i	= 1:N,
        ybuf(:,i)	=	ybuf(:,i+1);
     end 
          
     ybuf(:,N+1)		=	ymea(:, 1);

     
     
     for i	= 1:N+1,
        umvb(:,i)	=	umvb(:,i+1);
     end 
     
     umvb(:,N+2)		=	umv(:, 1); 
     
     
     %__________________________________________________________________________________
 
     %
     %__________________________________________________________________________________
     
     
     
     
     %__________________________________________________________________________________
     % 
     % Wainting until MHE reaches the Horizon Time= Ts*(N) 
     % In this phase (meanwhile t<(N+1)*dt) we won't predict Pk-N|k, only the correction
     % phase.
     
     if t<(N+1)*dt,
        disp('***************************************************************************')
        disp('Waiting until MHE reaches the Horizon Time= Ts*(N)')
        disp('***************************************************************************')
        disp(' ')
        
        
        
        %__________________________________________________________________________________
        %
        % Step 1: Perform only the prediction phase from t=0 to t= 1*dt! 
        % Here we will obtain the x{k+1|k} value only by calling pmfun. 
        % xp and Pp are the initial valeus for the process or system.
        
        
        if t==0,  	% 		for t= 0*dt or N=0
           
           disp('***************************************************************************')
           disp('Special case, N=0 --> We got a cEKF problem !!!! ')
           disp('***************************************************************************')
           disp(' ')
           
           N					=	0						;
           parmhe.N			=	N						;
           
           xsm					=	xp(:,1)				;
           ym					=	ybuf(:,end)		;
           um					=	umv					;
           
                    
           %__________________________________________________________________________________
           %
           % Step 2:  here the algorithm will perform a biased optimization problem.
           % It means that we will assume a modified MHE optimization problem, where we will
           % find an optimal value for x{k|k}, with arrival cost equal to zero, based on 
           % the following :
           %
           % min {xk|k} f = w'Qw + v'Rv.
           % 
           % w =	x{k|k} - 	x{k|k-1};
           % v =	ymea 	- 	g(xk|k);
           %
           % from optimization problem results we will obtain the x{x|x}cor and P{k-N}cor
           % and after we will only predict the x{x+1|x}
           
        else  % t>=1*dt & t< (N+1)*dt,
           
                      
           % Here we can perform a full MHE optimization problem, because it has           
           % reached the minimum MHE horizon N=1, and we can continue to perform until
           % this reaches the Horizon time t=N*dt.
           % Note 1: here we will also perform the MHE correction phase, so we need to set 
           % some statements in order to get the right values from the buffer, because 
           % the horizon time t=(N+1)*dt wasn't reached yet.
           % 
                      
           n					=	t/dt								;
           N					=	n									;
           parmhe.N			=	N									;
           
           
                      
           
           % getting x values from the buffer
           xsm				=	xbuf(:,(end-N:end))			;
           ym				=	ybuf(:,(end-N:end))			;
           um				=	umvb(:,(end-N:end))			;
                                                   
        end
        
     else
        
        % for t>= (N+1)*dt
        
        if t==(N+1)*dt,
           disp('***************************************************************************')
           disp('...finished waiting, now running MHE!!!')
           disp('***************************************************************************')
           disp(' ')
        end
        
        N					=	parmhe.N		;
        
        xsm 			=	xbuf				;
        ym 				=	ybuf				;
        parpro.umv	= 	umvb(:,2)		;					% um{t(k-N)}
        um				=	umvb				;
        
     end
     
     
     % #################################################################################
     % #########################                ########################################
     % #########################   CORRECTION   ########################################
     % #########################                ########################################
     % #################################################################################
        
               
     MHE_c	;   				% Runs the Correction phase by MHE
     
     % 							Results from the MHE Correction Phase:
     %
     %							{x_k-N|k, x_k-N+1|k, ... x_k|k} and Pk-N|k
     
     % #################################################################################
     % #########################                ########################################
     % #########################   PREDICTION   ########################################
     % #########################                ########################################
     % #################################################################################
     
     
     % disp(' ')
     % disp('... predicting the x(k+1|k) and  P(k-N+1|k) states  from x(k|k) and P(k-N|k)') 
     % disp(' ')          
     
     %__________________________________________________________________________________
     % Runs the Prediction phase by cEKF
     
     CEKF_p	;  
     
     % 							Results from the cEKF Prediction Phase:
     %
     %							{x_k-N+1|k}pred ,  Pk-N+1|k and {x_k+1|k}
     %
     %__________________________________________________________________________________
     
          
     disp(' ')
     disp(' ')
     if t==Tsim,
        disp('***************************************************************************')
        disp('**************    Finished simulation on MHE !!!!    **********************')
        disp('***************************************************************************')
     else
     end
     
     
     
     %__________________________________________________________________________________
     %
     %                         Set parameters/values on buffer
     %
     %
     %     
     %    
     %  Manipulated variables, size=(nmu, N+1)
     %

     in1					=	umvb;
     
     %     
     %  xstart
     %
     
     N 			= 	get_param('MHE_CEKF/Horizon', 'Value');
     N	 		=	str2num(N);
     
     
     if t==0*dt
        xbuf(:,end) = xcor(:,end)			;
        
     elseif t>=1*dt & t< (N+1)*dt
        xbuf(:,(end-n):end) = xcor(:,(end-n):end)	;
     else
        xbuf 			= xcor						;
     end
     
     for i	= 1:N,
        xbuf(:,i)	=	xbuf(:,i+1); 
     end 
          
     xbuf(:,N+1)		=	x_k1(1:ns, 1); 
     
      xbuf
          
              
     %                    
     
     in2					=	xbuf;
     
     %  ystart
     %
     in3					=	ybuf;
     %
     
     
     %  Filtered state x { k-N-1|k-1} = xfil
     %
     %
     in4				=	xkNcor;

     
         
     setpar;
     
     % t, P_kN1
     %__________________________________________________________________________________

    
sys=[	x_k1(:);		P_kN1(:)	];
% 			x_k+1|k			Pk-N+1|k 	]     
     
  %%%%%%%%%%%
  % Outputs %
  %%%%%%%%%%%
  case 3,
     sys=x(1:ns,1);
     
     

  %%%%%%%%%%%%%%%%%%%%%%%
  % GetTimeOfNextVarHit %
  %%%%%%%%%%%%%%%%%%%%%%%
  case 4,
    sys=[] % mdlGetTimeOfNextVarHit(t,x,u);

  %%%%%%%%%%%%%
  % Terminate %
  %%%%%%%%%%%%%
  case 9,
    sys=[]; % do nothing

  %%%%%%%%%%%%%%%%%%%%
  % Unexpected flags %
  %%%%%%%%%%%%%%%%%%%%
  otherwise
    error(['Unhandled flag = ',num2str(flag)]);

end


