%
% ################           MHE Correction        ####################
%
% ##################      4 CYLINDRICAL TANKS      #################### 
%
% By Giovani Tonel (giotonel@enq.ufrgs.br) - May 2007





%___________________________    OPTIMIZATION   ____________________________________
%
%                                 


% ----------------------------- Optimization Sets ---------------------------------
% Optimization Sets

opt 			= optimset( 'LevenbergMarquardt','on' ,'LargeScale', 'off',...
   'MaxFunEvals', 100000, 'MaxIter', 100000, 'TolX',1.5e-9);  

% ---------------------------------------------------------------------------------



% ----------------------   Check the objective function       ---------------------
% 
% 
% 
% f 		= 	MHE_objfun(xb0, t,[],xfil, ym,umvb,Pp, parmhe, parpro)
% ---------------------------------------------------------------------------------


%----------------------------------------------------------------------------------

Obj					=	'MHE_objfun'	;

% creating a best guess for the optimization problem

if t==0*dt,
   w0						=	diag(sqrt(Q))*ones(1,N+1)	;		% size= (ns, N+1)
   v0						=	diag(sqrt(R))*ones(1,N+1)	;		% size= (nms, N+1)
   
   wv0 						= 	w0(:);
   v0						=	v0(:);
   wv0					=	wv0(:);
   
   for i= ns*(N+1)+1 :(ns+nms)*(N+1),
      wv0(i)			=	v0(i-ns*(N+1)) ;
   end
else
     
   wv0 				= zeros(1,(N+1)*(ns+nms))*0.5;
end

% length(wv0)


optimizer 		= 	1;

% Optimization time 
% tic
%----------------------------------------------------------------------------------   






switch optimizer
case 1
   %														FMINCON
   %
   % [xopt]			=	fmincon(FUN,X0,A,B,Aeq,Beq,LB,UB,NONLCON,OPTIONS,P1,P2,...)   
   
            
   [wvopt, f]		=	fmincon(Obj,wv0,[],[],[],[],[],[],'constraints',opt,...
      t,[],xfil,ym,um,Pp, parmhe, parpro);
      
         
					
   % end
   

   % xopt = {x_k-N|k, x_k-N+1|k, ... x_k|k} ,  size=(ns,N+1)
   
     
   
case 2
   %														HKJEEVES
   % [xopt,Opt,nS]	=	hkjeeves(Sx,guess,ip,Lb,Ub,problem,tol,mxit,stp,amp,red,par1, par2,...)
   %
   %														BUSCARND
   % [xopt,Ot,nS]	=	buscarnd(S,x0,ip,nOt,samples,Lb,Ub,problem,tol,mxit,R,red,mem,p1, p2,...)
   
   nOt						=	2			;
   tol						=	1e-6		;	% tolerance (default = 1e-4)
   mxit						=	1000		; 	% maximum number of stages (default = 50*(1+4*~(ip>0)))
   mem						=	50			; 	% mem: number of stored points during the exploration phase [0, min(18,samples-1)]
   red						=	0.5		; 	% red: factor to reduce the size of the axis vector (0,1) (default = 0.2)
   sa						=	20			; 	% samples: number of samples per stage (default = 3*size(x0(:),1))
   
      
   [wvopt, f]			= hkjeeves(Obj,wv0, 0, [], [], -1, tol, mxit,[], [], red,...
      t,[],xfil,ym,um,Pp, parmhe, parpro);
   
   % [xopt,f]			=	buscarnd(Obj,xb0, 0,nOt,[],Lb,Ub,-1,tol,mxit,[],red,mem,t,[],xfil, ym,umvb,Pp, parmhe, parpro)
   
   
   
case 3
   %														FMINUNC
   [xopt,f]				=	fminunc(Obj,xb0,opt,...
      t,[],xfil,xsm,ym,um,Pp, parmhe, parpro);  
   
   
   
case 4
   %														LSQNONLIN
   % [xopt]				=	lsqnonlin(FUN,X0,LB,UB,OPTIONS,P1,P2,..)
   [xopt, f]			=	lsqnonlin(Obj,xb0,Lb,Ub,opt,...
      t,[],xfil,ym,um,Pp, parmhe, parpro );
   
end


wopt				=	wvopt(1: ns*(N+1));		
vopt				=	wvopt(ns*(N+1)+1: (ns+nms)*(N+1));	

wopt				=	reshape(wopt, ns , N+1);		% size= (ns  , N+1 )
vopt				=	reshape(vopt, nms, N+1);		% size= (nms , N+1 )


xcor 		= 	xsm  + wopt;



% sprintf('... objective function value: fmincon= %0.4f  fhkjeeves= %0.4f ', fm, fhk)

% disp('... optimized variable xopt:')
% xopt;

sprintf('... objective function value (t= %0.1f)  : %0.4f ',t, f)



%__________________________   END OPTIMIZATION  ___________________________________

% toc



% ==========================   State Correcting =================================== 

% 			If the variables that were optimizated were the states disturbance, 
% 			w* = {w_k-N-1|k, w_k-N|k, ..., w_k-1|k}, then we ought to add the optimal
% 			state disturbance on the states, calculated previously, to obtain the corre-
%			cted state value:
% 									xcor				=	xp 				+ 	w_opt;
%										||					||							||
% 									x_k-N|k 		= x_k-N|k-1 	+ 	w_k-N-1|k
% 									x_k-N+1|k 	= x_k-N+1|k-1 	+ 	w_k-N|k
% 										.				.		.				.			.
% 									x_k|k 			= x_k|k-1 		+ 	w_k-1|k
%
%
% 			If the variables that were optimizated were the own states, 
% 			xopt		=	{x_k-N|k, x_k-N+1|k, ... x_k|k}, then we need only get their values 
%			from the optimization problem output:
%
% 									xopt = x*	= xcor	=	{x_k-N|k, x_k-N+1|k, ... x_k|k}, size=(ns,N+1)

% ------------------------------    Getting x_k|k ---------------------------------
%
% x_k|k will be used to predicte x_k+1|k state value on the prediction phase (t_k).
x_kk			=	xcor(:,end);
% ---------------------------------------------------------------------------------

% ------------------------------    Getting x_k-N|k -------------------------------
%
% Now x_k-N|k will be a filtered state (xfil) and he will be used to predicte x_k-N+1 
% state value on the next correction phase (t_k+1).
%
xkNcor			=	xcor(:,1); 
% xfil			=	xcor(:,1); 
%
% it will be updated on the Set parameters phase in the MHE.m file.
% ---------------------------------------------------------------------------------


% -----------------------    Correcting F Jacobian Matrix -------------------------
% F_{k-N|k}cor = f[x{k-N|k}cor, u{k-N|k} ]
%
F 				= Fmatrix(xkNcor,parpro);
% ---------------------------------------------------------------------------------

% ------------------------  Correcting G Jacobian Matrix --------------------------
% G_{k-N|k}cor = f[x{k-N|k}cor} ]
%
G				= Gmatrix(xkNcor,parpro);
% ---------------------------------------------------------------------------------

     
% ------------------------- Up-dating P gain matrix  ------------------------------
%
% There are some forms of the update the P matrix, for this case we decided use this:

% P_k-N|k = 	Q + F{N|k}*P{k-N|k-1}-*F{N|k}'-F{N|k}*P{k-N|k-1}*G{N|k}'*inv(G{N|k}*
% 						P{k-N|k-1}*G{N|k}'+R)*G{N|k}*P{k-N|k-1}*F{N|k}';

PkNcor 		= 	Q + F*Pp*F' - F*Pp*G'*inv(G*Pp*G' + R)*G*Pp*F';
% ---------------------------------------------------------------------------------
