function [c,	ceq]= constraints(wv,t,options,xfil, ymea, umvb,P_kN, parmhe, parpro)
% 
% ##################         CONSTRAINTS                  ##################
%
% ##################   MOVING HORIZON ESTIMATOR (MHE)     ##################
%
%___________________________________________________________________________
%___________________________________________________________________________
%
%             Giovani Tonel (giotonel@enq.ufrgs.br) - May 2007
%___________________________________________________________________________
%___________________________________________________________________________
%
% Set constraints.
% 		X=FMINCON(FUN,X0,A,B,Aeq,Beq,LB,UB,NONLCON) subjects the minimization to the 
%   	constraints defined in NONLCON. The function NONLCON should return the vectors
%    C and Ceq, representing the nonlinear inequalities and equalities respectively, 
%    when called with feval: [C, Ceq] = feval(NONLCON,X). FMINCON minimizes
%    FUN such that C(X)<=0 and Ceq(X)=0. (Set LB=[] and/or UB=[] if no bounds exist.)

% 		C(X) <= 0, Ceq(X) = 0   (nonlinear constraints)
%                                 LB <= X <= UB     

% ---------------------------------------------------------------------------------
%

% S-function name
namesf		= 1;

%  Convert field to variable
par_pro;
% ---------------------------------------------------------------------------------


% -----------------------  Reshape guess data from the optimizer ------------------

wopt			=	wv(1: ns*(N+1));		
vopt			=	wv(ns*(N+1)+1: (ns+nms)*(N+1));	

wopt			=	reshape(wopt, ns , N+1);		% size= (ns  , N+1 )
vopt			=	reshape(vopt, nms, N+1);		% size= (nms , N+1 )

						
% ---------------------------------------------------------------------------------


%__________________________________________________________________________________
%__________________________________________________________________________________
%
%                               MHE WINDOW
%__________________________________________________________________________________

xpred(1:ns,1) 					= xfil ;

for i=N+1:-1:1,
   % 												 			% i= N+1		,	i= N			, ... , i=1
   % --------------------------- Time Horizon integration  ---------------------------
   %																			|					|				|			|
   % i= N+1		,	i= N			, ... , i=1
   ti 						= 	t-(i)*dt		;				% t(k-N-1)	,	t(k-N)		, ... , t(k-1)
   tf						=	t-(i-1)*dt	;				% t(k-N)		,	t(k-N+1	)	, ... , t(k)
   delt						=	[ti , tf]		;				% [initial t, final t]
   % ---------------------------------------------------------------------------------
   
        
   % ------- Getting the manipulated variables from the buffer    --------------------
   % 		umvb{ k-N-1|k, k-N|k, . . ., k|k}, size=(nmu, N+2)
   
   parpro.umv				= 	umvb(:,N+2-i);
   
   % ---------------------------------------------------------------------------------
   
   if N==0,
      
      % when t==0*dt, the horizon length is equal to zero (N= 0)
      % x{k|k}				=	x{k|0} + w{k|0} =	x0 + w0;
      
      xpred(1:ns,N+2-i)	=	xpred(1:ns,N+2-i)+ wopt(:,N+2-i);
   else
      if N~=0 & (t/dt)<N+1,
         
         if i==N+1,
            xpred(1:ns,N+2-i)	=	xpred(1:ns,N+2-i)+ wopt(:,N+2-i);
         else
            % 
            % x{k+1|k}				=	f(x{k|k} + w{k|k})
            
            xpred(1:ns,N+2-i)	=	xpred(1:ns,N+2-i)  +  wopt(:,N+2-i);
            Ypred						= pmfun(xpred(1:ns,N+2-i), delt, parpro );
            xpred(1:ns,N+2-i)	=	Ypred(1:ns)';
         end
      else
         xpred(1:ns,N+2-i)	=	xpred(1:ns,N+2-i)  +  wopt(:,N+2-i);
         Ypred						= pmfun(xpred(1:ns,N+2-i), delt, parpro );
         xpred(1:ns,N+2-i)	=	Ypred(1:ns)';
         
      end
   end
   
      
   yc(:,N+2-i)			=	smfun(xpred(1:ns,N+2-i) , parpro);
   
   ceqw						=	zeros(ns, N+2-i);  
   ceqv						=	yc(:,N+2-i)		- ( ymea(:,N+2-i) + vopt(:,N+2-i)	);
      
   xpred(1:ns,N+3-i)	=	xpred(1:ns,N+2-i);
   
end


ceq						=	[ceqw(:); ceqv(:) ]	;

c							=	[]	; 						% without nonlinear inequalities constraints
   