%
% ##################  GENERAL PARAMETERS (Initialization) ##################
%                                           
% ##################   MOVING HORIZON ESTIMATOR (MHE)     ##################
% ##############  CONSTRAINED EXTEND KALMAN FILTER (cEKF)    ###############
%
%___________________________________________________________________________
%___________________________________________________________________________
%
%             Giovani Tonel (giotonel@enq.ufrgs.br) - April 2007
%___________________________________________________________________________
%___________________________________________________________________________



clear all
clear functions
clc



disp(' ')
disp(' #############################################################################')
disp(' #############################################################################')
disp(' ###############################           ###################################')
disp(' ###############################  MHE/cEKF ###################################')
disp(' ###############################           ###################################')
disp(' #####################   Model:4 CYLINDRICAL TANKS   #########################')
disp(' #############################################################################')
disp(' #############################################################################')
disp(' ');
disp('          General Parameters for Initialization');
disp(' ');
disp(' _____________________________________________________________________________');

% help initialization_MHE_CEKF


%_________________________________________________________________________________
%
%                                   PARAMETERS
%_________________________________________________________________________________

% Info Processes

ns 		= 	get_param('MHE_CEKF/n_s', 'Value');
ns	 		=	str2num(ns);

nms 		= 	get_param('MHE_CEKF/n_ms', 'Value');
nms	 	=	str2num(nms);

% ns		=	4			;				% number of states
% nms	=	2			;				% number of measured states (filtered states).

%_________________________________________________________________________________


%_________________________________________________________________________________
%
% Initial state covariance, cm^2
%
P0			=		eye(ns)*(1e3);	
%_________________________________________________________________________________
%
%_________________________________________________________________________________
%
% Disturbance covariance, cm^2
% 
Q			= 		eye(ns)*(1);				% w ~ N(0,Q) meaning w is gaussian noise with covariance Q
%_________________________________________________________________________________	
%
%_________________________________________________________________________________
%
% Noise covariance, cm^2
% 
R			=		eye(nms)*(30);				% v ~ N(0,R) meaning v is gaussian noise with covariance R
%_________________________________________________________________________________	


Ts 		= 	1.0		;     		% Sampling Time		[min]
Tsim		= 	100		;     		% Simulation Time 	[min]
NA			=	1.0		; 				% Noise Amplitude 	[cm ]

H			=	[ns nms];
printmat(H,'Info Processes','Value','ns  nms')

if ns==nms,
   disp('The number of states is the same of the number of estimated states!')
end



flag1	=	1; 	% kind of system chosen, see matrixH.m
 						% (1) 	--> linear system
              	% (2) 	--> nonlinear system
              	% (3) 	--> nonlinear system (modified)
                 
flag2 	= 1;		% (1) 	--> measured level,
              	% (2) 	--> measured Fout
                 

% ################################################################################
%
% 													HORIZON SIZE
%
% ################################################################################


% The Horizon size can be settled in the Simulink block named "Horizon"
% N		= 2;
N 			= 	get_param('MHE_CEKF/Horizon', 'Value');
N	 		=	str2num(N);


% ################################################################################

% Horizon Time  
Th			=		Ts*(N+1);
H1			=		[Ts N Th];

printmat(H1,'MHE Sets','Value','Ts     N    H_time')

%_________________________________________________________________________________
%
%_________________________________________________________________________________


%_________________________________________________________________________________
%
% Initial states values (levels)

h1o		=		12.4;					% cm
h2o		=		12.7;					% cm
h3o		=		1.8;						% cm
h4o		=		1.4;						% cm

% initial conditions for the model process
xx0			= 		[h1o
   h2o 
   h3o 
   h4o];	

% initial conditions for the estimators the model process

x0				= 		[h1o+2
   h2o+2
   h3o 
   h4o];	

% ---------------   Initial states for the MHE horizon (ns, N+1) ------------------

xstart	=		ones(ns,N+1);

% xstart	= [
%   12.4000   12.4000   12.3323
%   12.7000   12.7000   12.6379
%   1.8000    1.7000    1.6756
%   1.4000    1.4000    1.3380 ];

% ------------------- Initial measurement  (nms, N+1) -----------------------------


% {yk-N, yk-N+1, yk-N+2,..., yk-1 } 

ystart	=	ones(nms,N+1);	

% ystart	=		[
%   12.4000   12.6000   12.2329
%   12.7000   12.9000   12.5384   	]	;

% Note: we need to set the x a y size in the simulink block before we start the simu-
% lation, because the Matlab forbides to reshape or change after simulation started.
% ---------------------------------------------------------------------------------



% State limits

xup		=		[];
xlow		=		[];

  

As 		=		[ 28 32 28 32]';           	% [cm2] cross section of each tank
a 			=		[ 0.071 0.057 0.071 0.057]';	% [cm2] cross section of the outlet hole of each tank
g 			= 		981;                        	% acceleration of gravity [cm/s2]

F10		=		8.2;											% Initial flow [cm^3/s]
F20		=		8.0;											% Initial flow [cm^3/s]
X10		=		0.7;											% X1 flow fraction
X20		=		0.6;											% X2 flow fraction


%_________________________________________________________________________________



%
% Generating, ploting and saving a PRBS signal
%


button = questdlg('Do you want generate and save a new PRBS signal version?', ...
   'PRBS signal','Yes','No','No');


switch button
case 'Yes',
   
   disp(' ')
   disp('...generating a PRBS signal');
   
   tsim 		= [0:Ts:Tsim/Ts]';
   F1_prbs 	= 7 + 4*prbs(Tsim + 1, 6, 6, 0.0, 1) ; 
   F2_prbs 	= 7 + 4*prbs(Tsim + 1, 4, 4, 0.8, 1) ;
   
   disp(' ')
   disp('...saving PRBS signal on the local path');
   
   save('prbs_signal', 'tsim','F1_prbs', 'F2_prbs')
   
   plot(tsim,F1_prbs, tsim,F2_prbs) 
   axis([0 Tsim 3 11])
   xlabel('Time [min]')
   ylabel('Flow [cm^3/min]')
   legend('F1_i_n_p_u_t','F2_i_n_p_u_t' )  
   
case 'No',
   disp(' ')
   disp('... loading the previous saved PRBS signal version!!')
   load('prbs_signal')
   quit cancel;
end



%___________________________________________________________________________________
%

% Set Simulink system and block parameters -Print to Constant Block
%
%

umv0		= 		[F1_prbs(1); F2_prbs(1); X10; X20]	;			% manipulated variables, initial value (umv0)
num		=		length(umv0)				;
umvb		=		umv0*ones(1, N+2)		;			% buffer of the manipulated variables (umvb)


% Manipulated variables (u_m)
% 
in1				= umvb;


%  xstart
%
in2				=	xstart;
%
%  ystart
%
in3				=	ystart;
%
%  Filtered state xfil { k-N-1|k-1}
%
in4				=	x0;
%


%_________________________________________________________________________________
%
%                          Input parameters to S-functions and blocks in simulink
%_________________________________________________________________________________



%__________________________________
%
%  Initial Parameters for MHE
%__________________________________


parmhe.N					=	N			;
parmhe.P0					=	P0			;
parmhe.Q					=	Q			;
parmhe.R					=	R			;

%__________________________________
%
%  Initial Parameters for cEKF
%__________________________________

parcekf.P0				=	P0			;
parcekf.Q					=	Q			;
parcekf.R					=	R			;

%__________________________________
%
% Process Parameters
%__________________________________

parpro.x0					=	x0			;
parpro.dt					=	Ts			;
parpro.Tsim				=	Tsim		;
parpro.ns					=	ns			;
parpro.nms				=	nms   	;
parpro.flag1			=	flag1 	; 
parpro.flag2			=	flag2  	;
parpro.As					=	As 		;
parpro.a					=	a  		;
parpro.g					=	g  		;

% parpro.F1				=	F10 		;
% parpro.F2				=	F20 		;
% parpro.X1				=	X10 		;
% parpro.X2				=	X20 		;

parpro.umv				=	umv0 	;

parpro.num				=	num		;
%___________________________________________________________________________________




%  X1 flow fraction
%
in5				=	X10;
value5			= 	[sprintf(' %.3f ',in5)];	
set_param('MHE_CEKF/X_1','Value',value5);
%
%
%  X2 flow fraction
%
in6				=	X20;
value6			= 	[sprintf(' %.3f ',in6)];			
set_param('MHE_CEKF/X_2','Value',value6);


setpar;

%___________________________________________________________________________________


% load prbs_signal;

% whos
%___________________________________________________________________________________


%__________________________________

%___________________________________________________________________________________



%___________________________________________________________________________________
%
%                                    Checking 
%___________________________________________________________________________________


check;

%____________________________________________________________________________________




disp(' ')
disp('********************** SELECTED OPTIONS *************************************');

switch flag1
case 1
   disp(' ')
   disp('- Linear System!!!');
   
case 2
   disp(' ')
   disp('- Nonlinear System!!!');
case 3
   disp(' ')
   disp('- Nonlinear System (modified)!!!');
end

                 
switch flag2
case 1
   disp(' ')
   disp('- y measured of process (h1 and h2) with white noise!!!');
   s_el 			=	[1 2]; 					% Selector Elements
case 2
   disp(' ')
   disp('- y measured of process (Fout1 and Fout2) with white noise!!!');
   s_el 			=	[3 4]; 					% Selector Elements
case 3
   disp(' ')
   disp('- other case');
end

value5			= 	[ sprintf('%i',ns)];
value6			= 	['[' sprintf(' %i ',s_el) ']' ];
set_param('MHE_CEKF/Selector','InputPortWidth',value5,'Elements',value6);

disp('*****************************************************************************');



disp(' ')
disp(' ')
disp(' ')
disp('*****************************************************************************');
disp(' ')
disp('************** Ok, now the MHE/cEKF is ready to run! ************************');
disp(' ')
disp('*****************************************************************************');
disp(' ')
disp(' ')
