function [sys,x0,str,ts] = process_4tq(t,x,u,flag,xx0,parpro)
%
% #######################      PROCESS MODEL        ################################
%
% #######################  4 CYLINDRICAL TANKS      ################################ 
%
% Giovani Tonel (giotonel@enq.ufrgs.br) - November 2006

%__________________________________
%
% Process Parameters


% S-function name
namesf		= 3;

%  Convert field to variable
par_pro;

%__________________________________



switch flag,

  %%%%%%%%%%%%%%%%%%
  % Initialization %
  %%%%%%%%%%%%%%%%%%
  case 0,
    sys = [ns,      				% number of continuous states
           0,      					% number of discrete states
           ns,      				% number of outputs
           num,      				% number of inputs
           0,      					% reserved must be zero
           1,      					% direct feedthrough flag
           1];     					% number of sample times
        
        
        
    x0  = [xx0];  					% initialize the initial conditions
    str = [];      					% str is always an empty matrix
    ts  = [0 0];   					% sample time: [period, offset]

  %%%%%%%%%%%%%%%
  % Derivatives %
  %%%%%%%%%%%%%%%
  
case 1,
   
   % ##########################  STATES  ########################################
   
    % h(1)			=	x(1);
    % h(2)			=	x(2);
    % h(3)			=	x(3);
    % h(4)			=	x(4);
    
    % h					=	x;
    
    % ##########################################################################
    
    %__________________________________________________________________________________
    %
    %                             INPUT PROCESS MEASUREMENT
    %
    %
    % 			Process inputs:									Process outputs:
    %
    % 		u(1) <-- F1	(feed flow)	;				% y(1) <-- h1 	;
    % 		u(2) <-- F2	(feed flow)	;				% y(2) <-- h2 	;
    % 		u(3) <-- X1	(fr. flow)	;				% y(3) <-- h3 	;
    % 		u(4) <-- X2	(fr. flow)	;				% y(4) <-- h4 	;  
    
    %
    % umv				:	 	u manipulated variables 
    % umv(1)		:		F1
    % umv(2)		:		F2
    % umv(3)		:		X1
    % umv(4)		:		X2
    
    
    umv							=	[u(1:num)];
    
    % Up-dating manipulated variables
    parpro.umv				=	umv 	;
    
    %___________________________________________________________________________
   
    
    %__________________________________________________________________________________
    
    [dxdt]= ODEpmfun(t,x, [], parpro);
    
    
    sys = 	[dxdt];	
    
    % #############################################################################

  %%%%%%%%%%
  % Update %
  %%%%%%%%%%
  case 2,
     sys=[];
     
  %%%%%%%%%%%
  % Outputs %
  %%%%%%%%%%%
  case 3,
    sys= [x];

  %%%%%%%%%%%%%%%%%%%%%%%
  % GetTimeOfNextVarHit %
  %%%%%%%%%%%%%%%%%%%%%%%
  case 4,
    sys=[] % mdlGetTimeOfNextVarHit(t,x,u);

  %%%%%%%%%%%%%
  % Terminate %
  %%%%%%%%%%%%%
  case 9,
    sys=[]; % do nothing

  %%%%%%%%%%%%%%%%%%%%
  % Unexpected flags %
  %%%%%%%%%%%%%%%%%%%%
  otherwise
    error(['Unhandled flag = ',num2str(flag)]);

end

